USE mrs;

DELIMITER //
DROP PROCEDURE IF EXISTS calcMetricFreq//
CREATE PROCEDURE calcMetricFreq ()
  COMMENT 'Dynamic statistical calculation for metric frequencies'

BEGIN
  DECLARE done INT DEFAULT 0;
  DECLARE roc1, roc2 VARCHAR(200);
  DECLARE flavour1, service1, metric1, time1 INT;
  DECLARE flavour2, service2, metric2, time2 INT;
  DECLARE cur CURSOR FOR
  SELECT get_roc_name(service_id) AS roc, flavour_id, metric_id, check_time, service_id
    FROM metricdata
    JOIN service ON metricdata.service_id = service.id
    WHERE check_time > (UNIX_TIMESTAMP() - 1209600) -- 2 weeks ago
      AND metricstatus_id IN (
          SELECT id FROM metricstatus
          WHERE description != "MISSING" 
            AND description != "REMOVED")
    ORDER BY roc, flavour_id, metric_id, service_id, check_time;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;
  
  DROP TEMPORARY TABLE IF EXISTS tmpMetricFreq;  
  CREATE TEMPORARY TABLE tmpMetricFreq 
  (roc VARCHAR(200), flavour_id INT, metric_id INT, freq INT)
  ENGINE = MEMORY;

  DELETE FROM mon_frequency_statistics;
  
  OPEN cur;
  REPEAT
    FETCH cur INTO roc1, flavour1, metric1, time1, service1;
    IF (flavour1 != flavour2) || (metric1 != metric2) THEN
      INSERT INTO mon_frequency_statistics 
      SELECT roc, flavour_id, metric_id, AVG(freq), STDDEV(freq) 
        FROM tmpMetricFreq
        GROUP BY roc, flavour_id, metric_id;
      TRUNCATE tmpMetricFreq;
    ELSE 
      IF (service1 = service2) THEN
        IF ( (time1-time2) > 
             (SELECT frequency*60/2 FROM metrics_in_supported_services
               WHERE service_id = service1 AND metric_id = metric1
               LIMIT 1)
           ) THEN
          INSERT INTO tmpMetricFreq VALUES (roc1, flavour1, metric1, time1-time2);
        END IF;
      END IF;
    END IF;
    SET roc2     = roc1;
    SET flavour2 = flavour1;
    SET service2 = service1;
    SET metric2  = metric1;
    SET time2    = time1;
  UNTIL done END REPEAT; 
  CLOSE cur;

  DROP TEMPORARY TABLE IF EXISTS tmpMetricFreq;  
END;
//
DELIMITER ;

USE mrs;

DELIMITER //
DROP PROCEDURE IF EXISTS list2table//
CREATE PROCEDURE list2table (
  IN inList VARCHAR(255),
  IN outTable CHAR(64)
) COMMENT 'Split list to temporary table'

BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);

  SET @s = CONCAT('DROP TEMPORARY TABLE IF EXISTS ', outTable);
  PREPARE stmt FROM @s; EXECUTE stmt;

  SET @s = CONCAT('CREATE TEMPORARY TABLE ', outTable,  ' (name VARCHAR(50) NOT NULL DEFAULT \'\' ) ENGINE = MEMORY');
  PREPARE stmt FROM @s; EXECUTE stmt;
  
  
  SET intFoundPos = INSTR(inList,',');
  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(inList, 1, intFoundPos-1);
    SET inList = REPLACE(inList, CONCAT(strElement,','), '');

    SET @s = CONCAT('INSERT INTO ', outTable, ' (name) VALUES (\'', strElement,'\')');
    PREPARE stmt FROM @s; EXECUTE stmt;

    SET intFoundPos = INSTR(inList,',');
  END WHILE;

  IF inList <> '' THEN
    SET @s = CONCAT('INSERT INTO ', outTable, ' (`name`) VALUES (\'', inList, '\')');
    PREPARE stmt FROM @s; EXECUTE stmt;
  END IF; 

END;
//
DELIMITER ;

USE mrs;

DELIMITER //
DROP PROCEDURE IF EXISTS getFreshMetrics//
CREATE PROCEDURE getFreshMetrics (
  IN profileNameList VARCHAR(255),
  IN NGINameList VARCHAR(255),
  IN serviceFlavourNameList VARCHAR(255),
  IN metricNameList VARCHAR(255),
  IN fqanList VARCHAR(255),
  IN devPercIn INT,
  IN devTime INT
) COMMENT 'Get missing metrics depending on variables'

BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE missingStatusId INT;
  DECLARE devPerc DECIMAL(3,2);

  SET devPerc = devPercIn/100 + 1;
   
  DROP TEMPORARY TABLE IF EXISTS tmpStatusList;  
  CREATE TEMPORARY TABLE tmpStatusList ENGINE = MEMORY
  SELECT id FROM metricstatus
  WHERE description != "MISSING" 
    AND description != "REMOVED";

  -- split profileNameList into temporary table
  SET @s = CONCAT('CALL list2table("', profileNameList,'","tmpProfileNameList")');
  PREPARE statement FROM @s; EXECUTE statement;
  -- if there is an ANY clause put everything into the table
  IF EXISTS (SELECT * FROM tmpProfileNameList WHERE name='ANY') THEN
    TRUNCATE TABLE tmpProfileNameList;
    INSERT INTO tmpProfileNameList SELECT name FROM profile;
  END IF;

  -- split NGINameList into temporary table
  SET @s = CONCAT('CALL list2table("', NGINameList,'","tmpNGINameList")');
  PREPARE statement FROM @s; EXECUTE statement;
  -- if there is an ANY clause put everything into the table
  IF EXISTS (SELECT * FROM tmpNGINameList WHERE name='ANY') THEN
    TRUNCATE TABLE tmpNGINameList;
    INSERT INTO tmpNGINameList SELECT DISTINCT get_roc_name(service_id) AS name FROM metrics_in_supported_services;
  END IF;
 
  -- split serviceFlavourNameList into temporary table
  SET @s = CONCAT('CALL list2table("', serviceFlavourNameList,'","tmpFlavourNameList")');
  PREPARE statement FROM @s; EXECUTE statement;
  -- if there is an ANY clause put everything into the table
  IF EXISTS (SELECT * FROM tmpFlavourNameList WHERE name='ANY') THEN
    TRUNCATE TABLE tmpFlavourNameList;
    INSERT INTO tmpFlavourNameList SELECT flavourname FROM service_type_flavour;
  END IF;

  -- split metricNameList into temporary table
  SET @s = CONCAT('CALL list2table("', metricNameList,'","tmpMetricNameList")');
  PREPARE statement FROM @s; EXECUTE statement;
  -- if there is an ANY clause put everything into the table
  IF EXISTS (SELECT * FROM tmpMetricNameList WHERE name='ANY') THEN
    TRUNCATE TABLE tmpMetricNameList;
    INSERT INTO tmpMetricNameList SELECT name FROM metric;
  END IF;

  -- split fqanList into temporary table
  SET @s = CONCAT('CALL list2table("', fqanList,'","tmpFqanList")');
  PREPARE statement FROM @s; EXECUTE statement;
  -- if there is an ANY clause put everything into the table
  IF EXISTS (SELECT * FROM tmpFqanList WHERE name='ANY') THEN
    TRUNCATE TABLE tmpFqanList;
    INSERT INTO tmpFqanList SELECT name FROM fqan;
    INSERT INTO tmpFqanList (`name`) VALUES ('NULL');
  END IF;

  -- create temp profile table with metric id and flavour id
  DROP TEMPORARY TABLE IF EXISTS tmpJoinProfile;
  CREATE TEMPORARY TABLE tmpJoinProfile ENGINE = MEMORY
  SELECT name, metric_id, service_type_flavour_id 
    FROM profile 
    JOIN profile_metric_map ON profile.id = profile_metric_map.profile_id
  WHERE name IN (SELECT name FROM tmpProfileNameList);
  
  -- create temp table for operations  
  DROP TEMPORARY TABLE IF EXISTS tmpFilteredMetrics;
  
  CREATE TEMPORARY TABLE tmpFilteredMetrics ENGINE = MEMORY
  SELECT get_roc_name(service.id) AS NGI,
	  service_type_flavour.flavourname AS flavour_name, 
	  metric.name AS metric_name, 
 	  IFNULL(fqan.name,'NULL') AS fqan_name,
 	  COUNT(*) AS counter
	  FROM metricdata_latest 
  JOIN metric ON metricdata_latest.metric_id = metric.id
  JOIN service ON metricdata_latest.service_id = service.id 
  JOIN service_type_flavour ON service.flavour_id = service_type_flavour.id
  LEFT JOIN fqan ON metricdata_latest.fqan_id = fqan.id
  WHERE metricstatus_id IN (SELECT id FROM tmpStatusList)
    AND service_type_flavour.flavourname IN (SELECT name FROM tmpFlavourNameList)
    AND metric.name IN (SELECT name FROM tmpMetricNameList)
    AND metric.id IN (SELECT metric_id FROM tmpJoinProfile WHERE service_type_flavour_id = service.flavour_id)
    AND check_time > (  
             SELECT DISTINCT CASE
              WHEN ( frequency*60*devPerc < IFNULL( (freq_mean+2*freq_std),frequency*60*devPerc) &&
                     IFNULL( (freq_mean+2*freq_std),frequency*60*devPerc) < 2*frequency*60) THEN (UNIX_TIMESTAMP() - (freq_mean+2*freq_std+devTime*60) )
              WHEN ( IFNULL( (freq_mean+2*freq_std),frequency*60*devPerc) <= frequency*60*devPerc) THEN (UNIX_TIMESTAMP() - (frequency*devPerc+devTime)*60 )
              ELSE ( UNIX_TIMESTAMP() - (2*frequency+devTime)*60 )
              END AS freq
               FROM mon_frequency_statistics
               RIGHT JOIN metrics_in_supported_services
                 ON mon_frequency_statistics.metric_id = metrics_in_supported_services.metric_id
                AND mon_frequency_statistics.roc = get_roc_name(metrics_in_supported_services.service_id)
               JOIN service AS atps
                 ON metrics_in_supported_services.service_id = atps.id 
              WHERE metrics_in_supported_services.metric_id = metricdata_latest.metric_id
                AND atps.flavour_id = service.flavour_id
                AND get_roc_name(metrics_in_supported_services.service_id) = get_roc_name(service.id) LIMIT 1
          )
  GROUP BY NGI, flavour_name, metric_name, fqan_name
  HAVING fqan_name IN (SELECT name FROM tmpFqanList); 
  
-- SELECT * FROM tmpFilteredMetrics;

  DROP TEMPORARY TABLE IF EXISTS tmpMetricsCount;
  
  CREATE TEMPORARY TABLE tmpMetricsCount ENGINE = MEMORY
  SELECT get_roc_name(service.id) AS NGI,
    service_type_flavour.flavourname AS flavour_name,
    metric.name AS metric_name,
    IFNULL(fqan.name,'NULL') AS fqan_name,
    count(*) AS counter
    FROM metrics_in_supported_services
    JOIN service ON metrics_in_supported_services.service_id = service.id
    JOIN service_type_flavour ON service.flavour_id = service_type_flavour.id 
    JOIN metric ON metrics_in_supported_services.metric_id = metric.id
    LEFT JOIN fqan ON metrics_in_supported_services.fqan_id = fqan.id
    WHERE service_type_flavour.flavourname IN (SELECT name FROM tmpFlavourNameList)
      AND metric.id IN (SELECT metric_id FROM tmpJoinProfile WHERE service_type_flavour_id = service.flavour_id)
      AND metric.name IN (SELECT name FROM tmpMetricNameList)
  GROUP BY NGI, flavour_name, metric_name, fqan_name
  HAVING fqan_name IN (SELECT name FROM tmpFqanList);

-- SELECT * FROM tmpMetricsCount;

   SELECT tmpMetricsCount.NGI AS NGI,
     tmpMetricsCount.flavour_name AS flavour_name,
     tmpMetricsCount.metric_name AS metric_name,
     tmpMetricsCount.fqan_name AS fqan_name,
     IFNULL(tmpFilteredMetrics.counter,0) AS received,
     tmpMetricsCount.counter AS total,
     IFNULL(CEILING((tmpFilteredMetrics.counter / tmpMetricsCount.counter)*100),0) AS percentage
   FROM tmpMetricsCount
     LEFT JOIN tmpFilteredMetrics ON tmpFilteredMetrics.NGI = tmpMetricsCount.NGI
           AND tmpFilteredMetrics.flavour_name = tmpMetricsCount.flavour_name
           AND tmpFilteredMetrics.metric_name = tmpMetricsCount.metric_name
           AND tmpFilteredMetrics.fqan_name = tmpMetricsCount.fqan_name
     WHERE tmpMetricsCount.NGI IN (SELECT name FROM tmpNGINameList);

END;
//
DELIMITER ;

